using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman1 : FeynmanDescriptor {
    private readonly int trainingSamples;
    private readonly int testSamples;

    public override string Name {
      get { return string.Format("Feynman I.6.2a exp(-theta**2/2)/sqrt(2*pi) | {0} samples", trainingSamples); }
    }

    protected override string TargetVariable { get { return "f"; } }
    protected override string[] VariableNames { get { return new[] {"theta", "f"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"theta"}; } }

    public int Seed { get; private set; }

    public Feynman1() : this((int) DateTime.Now.Ticks, 10000, 10000) { }

    public Feynman1(int seed) {
      Seed            = seed;
      trainingSamples = 10000;
      testSamples     = 10000;
    }

    public Feynman1(int seed, int trainingSamples, int testSamples) {
      Seed                 = seed;
      this.trainingSamples = trainingSamples;
      this.testSamples     = testSamples;
    }

    protected override int TrainingPartitionStart { get { return 0; } }
    protected override int TrainingPartitionEnd { get { return trainingSamples; } }
    protected override int TestPartitionStart { get { return trainingSamples; } }
    protected override int TestPartitionEnd { get { return trainingSamples + testSamples; } }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data  = new List<List<double>>();
      var theta = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 3).ToList();

      var f = new List<double>();

      data.Add(theta);
      data.Add(f);

      for (var i = 0; i < theta.Count; i++) {
        var res = Math.Exp(Math.Pow(-theta[i], 2) / 2) / Math.Sqrt(2 * Math.PI);
        f.Add(res);
      }

      return data;
    }
  }
}