using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman12 : FeynmanDescriptor {
    private readonly int testSamples;
    private readonly int trainingSamples;

    public Feynman12() : this((int) DateTime.Now.Ticks, 10000, 10000) { }

    public Feynman12(int seed) {
      Seed            = seed;
      trainingSamples = 10000;
      testSamples     = 10000;
    }

    public Feynman12(int seed, int trainingSamples, int testSamples) {
      Seed                 = seed;
      this.trainingSamples = trainingSamples;
      this.testSamples     = testSamples;
    }

    public override string Name { get { return string.Format("Feynman I.12.5 q2*Ef | {0} samples", trainingSamples); } }

    protected override string TargetVariable { get { return "F"; } }
    protected override string[] VariableNames { get { return new[] {"q2", "Ef", "F"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"q2", "Ef"}; } }

    public int Seed { get; private set; }

    protected override int TrainingPartitionStart { get { return 0; } }
    protected override int TrainingPartitionEnd { get { return trainingSamples; } }
    protected override int TestPartitionStart { get { return trainingSamples; } }
    protected override int TestPartitionEnd { get { return trainingSamples + testSamples; } }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data = new List<List<double>>();
      var q2   = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var Ef   = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var F = new List<double>();

      data.Add(q2);
      data.Add(Ef);
      data.Add(F);

      for (var i = 0; i < q2.Count; i++) {
        var res = q2[i] * Ef[i];
        F.Add(res);
      }

      return data;
    }
  }
}