using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman46 : FeynmanDescriptor {
    private readonly int testSamples;
    private readonly int trainingSamples;

    public Feynman46() : this((int) DateTime.Now.Ticks, 10000, 10000) { }

    public Feynman46(int seed) {
      Seed            = seed;
      trainingSamples = 10000;
      testSamples     = 10000;
    }

    public Feynman46(int seed, int trainingSamples, int testSamples) {
      Seed                 = seed;
      this.trainingSamples = trainingSamples;
      this.testSamples     = testSamples;
    }

    public override string Name {
      get { return string.Format("Feynman I.43.31 mob*kb*T | {0} samples", trainingSamples); }
    }

    protected override string TargetVariable { get { return "D"; } }
    protected override string[] VariableNames { get { return new[] {"mob", "T", "kb", "D"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"mob", "T", "kb"}; } }

    public int Seed { get; private set; }

    protected override int TrainingPartitionStart { get { return 0; } }
    protected override int TrainingPartitionEnd { get { return trainingSamples; } }
    protected override int TestPartitionStart { get { return trainingSamples; } }
    protected override int TestPartitionEnd { get { return trainingSamples + testSamples; } }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data = new List<List<double>>();
      var mob  = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var T    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var kb   = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var D = new List<double>();

      data.Add(mob);
      data.Add(T);
      data.Add(kb);
      data.Add(D);

      for (var i = 0; i < mob.Count; i++) {
        var res = mob[i] * kb[i] * T[i];
        D.Add(res);
      }

      return data;
    }
  }
}