using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman53 : FeynmanDescriptor {
    private readonly int testSamples;
    private readonly int trainingSamples;

    public Feynman53() : this((int) DateTime.Now.Ticks, 10000, 10000) { }

    public Feynman53(int seed) {
      Seed            = seed;
      trainingSamples = 10000;
      testSamples     = 10000;
    }

    public Feynman53(int seed, int trainingSamples, int testSamples) {
      Seed                 = seed;
      this.trainingSamples = trainingSamples;
      this.testSamples     = testSamples;
    }

    public override string Name {
      get { return string.Format("Feynman II.3.24 Pwr/(4*pi*r**2) | {0} samples", trainingSamples); }
    }

    protected override string TargetVariable { get { return "flux"; } }
    protected override string[] VariableNames { get { return new[] {"Pwr", "r", "flux"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"Pwr", "r"}; } }

    public int Seed { get; private set; }

    protected override int TrainingPartitionStart { get { return 0; } }
    protected override int TrainingPartitionEnd { get { return trainingSamples; } }
    protected override int TestPartitionStart { get { return trainingSamples; } }
    protected override int TestPartitionEnd { get { return trainingSamples + testSamples; } }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data = new List<List<double>>();
      var Pwr  = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var r    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var flux = new List<double>();

      data.Add(Pwr);
      data.Add(r);
      data.Add(flux);

      for (var i = 0; i < Pwr.Count; i++) {
        var res = Pwr[i] / (4 * Math.PI * Math.Pow(r[i], 2));
        flux.Add(res);
      }

      return data;
    }
  }
}