using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman54 : FeynmanDescriptor {
    private readonly int testSamples;
    private readonly int trainingSamples;

    public Feynman54() : this((int) DateTime.Now.Ticks, 10000, 10000) { }

    public Feynman54(int seed) {
      Seed            = seed;
      trainingSamples = 10000;
      testSamples     = 10000;
    }

    public Feynman54(int seed, int trainingSamples, int testSamples) {
      Seed                 = seed;
      this.trainingSamples = trainingSamples;
      this.testSamples     = testSamples;
    }

    public override string Name {
      get { return string.Format("Feynman II.4.23 q/(4*pi*epsilon*r) | {0} samples", trainingSamples); }
    }

    protected override string TargetVariable { get { return "Volt"; } }
    protected override string[] VariableNames { get { return new[] {"q", "epsilon", "r", "Volt"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"q", "epsilon", "r"}; } }

    public int Seed { get; private set; }

    protected override int TrainingPartitionStart { get { return 0; } }
    protected override int TrainingPartitionEnd { get { return trainingSamples; } }
    protected override int TestPartitionStart { get { return trainingSamples; } }
    protected override int TestPartitionEnd { get { return trainingSamples + testSamples; } }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data    = new List<List<double>>();
      var q       = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var epsilon = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var r       = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var Volt = new List<double>();

      data.Add(q);
      data.Add(epsilon);
      data.Add(r);
      data.Add(Volt);

      for (var i = 0; i < q.Count; i++) {
        var res = q[i] / (4 * Math.PI * epsilon[i] * r[i]);
        Volt.Add(res);
      }

      return data;
    }
  }
}