using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman75 : FeynmanDescriptor {
    private readonly int testSamples;
    private readonly int trainingSamples;

    public Feynman75() : this((int) DateTime.Now.Ticks, 10000, 10000) { }

    public Feynman75(int seed) {
      Seed            = seed;
      trainingSamples = 10000;
      testSamples     = 10000;
    }

    public Feynman75(int seed, int trainingSamples, int testSamples) {
      Seed                 = seed;
      this.trainingSamples = trainingSamples;
      this.testSamples     = testSamples;
    }

    public override string Name {
      get { return string.Format("Feynman II.34.2a q*v/(2*pi*r) | {0} samples", trainingSamples); }
    }

    protected override string TargetVariable { get { return "I"; } }
    protected override string[] VariableNames { get { return new[] {"q", "v", "r", "I"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"q", "v", "r"}; } }

    public int Seed { get; private set; }

    protected override int TrainingPartitionStart { get { return 0; } }
    protected override int TrainingPartitionEnd { get { return trainingSamples; } }
    protected override int TestPartitionStart { get { return trainingSamples; } }
    protected override int TestPartitionEnd { get { return trainingSamples + testSamples; } }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data = new List<List<double>>();
      var q    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var v    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var r    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var I = new List<double>();

      data.Add(q);
      data.Add(v);
      data.Add(r);
      data.Add(I);

      for (var i = 0; i < q.Count; i++) {
        var res = q[i] * v[i] / (2 * Math.PI * r[i]);
        ;
        I.Add(res);
      }

      return data;
    }
  }
}