using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman88 : FeynmanDescriptor {
    private readonly int testSamples;
    private readonly int trainingSamples;

    public Feynman88() : this((int) DateTime.Now.Ticks, 10000, 10000) { }

    public Feynman88(int seed) {
      Seed            = seed;
      trainingSamples = 10000;
      testSamples     = 10000;
    }

    public Feynman88(int seed, int trainingSamples, int testSamples) {
      Seed                 = seed;
      this.trainingSamples = trainingSamples;
      this.testSamples     = testSamples;
    }

    public override string Name {
      get { return string.Format("Feynman III.7.38 2*mom*B/(h/(2*pi)) | {0} samples", trainingSamples); }
    }

    protected override string TargetVariable { get { return "omega"; } }
    protected override string[] VariableNames { get { return new[] {"mom", "B", "h", "omega"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"mom", "B", "h"}; } }

    public int Seed { get; private set; }

    protected override int TrainingPartitionStart { get { return 0; } }
    protected override int TrainingPartitionEnd { get { return trainingSamples; } }
    protected override int TestPartitionStart { get { return trainingSamples; } }
    protected override int TestPartitionEnd { get { return trainingSamples + testSamples; } }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data = new List<List<double>>();
      var mom  = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var B    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var h    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var omega = new List<double>();

      data.Add(mom);
      data.Add(B);
      data.Add(h);
      data.Add(omega);

      for (var i = 0; i < mom.Count; i++) {
        var res = 2 * mom[i] * B[i] / (h[i] / (2 * Math.PI));
        omega.Add(res);
      }

      return data;
    }
  }
}