using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman92 : FeynmanDescriptor {
    private readonly int testSamples;
    private readonly int trainingSamples;

    public Feynman92() : this((int) DateTime.Now.Ticks, 10000, 10000) { }

    public Feynman92(int seed) {
      Seed            = seed;
      trainingSamples = 10000;
      testSamples     = 10000;
    }

    public Feynman92(int seed, int trainingSamples, int testSamples) {
      Seed                 = seed;
      this.trainingSamples = trainingSamples;
      this.testSamples     = testSamples;
    }

    public override string Name {
      get { return string.Format("Feynman III.12.43 n*(h/(2*pi)) | {0} samples", trainingSamples); }
    }

    protected override string TargetVariable { get { return "L"; } }
    protected override string[] VariableNames { get { return new[] {"n", "h", "L"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"n", "h"}; } }

    public int Seed { get; private set; }

    protected override int TrainingPartitionStart { get { return 0; } }
    protected override int TrainingPartitionEnd { get { return trainingSamples; } }
    protected override int TestPartitionStart { get { return trainingSamples; } }
    protected override int TestPartitionEnd { get { return trainingSamples + testSamples; } }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data = new List<List<double>>();
      var n    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var h    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var L = new List<double>();

      data.Add(n);
      data.Add(h);
      data.Add(L);

      for (var i = 0; i < n.Count; i++) {
        var res = n[i] * (h[i] / (2 * Math.PI));
        L.Add(res);
      }

      return data;
    }
  }
}