using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class FeynmanBonus5 : FeynmanDescriptor {
    private readonly int testSamples;
    private readonly int trainingSamples;

    public FeynmanBonus5() : this((int) DateTime.Now.Ticks, 10000, 10000) { }

    public FeynmanBonus5(int seed) {
      Seed            = seed;
      trainingSamples = 10000;
      testSamples     = 10000;
    }

    public FeynmanBonus5(int seed, int trainingSamples, int testSamples) {
      Seed                 = seed;
      this.trainingSamples = trainingSamples;
      this.testSamples     = testSamples;
    }

    public override string Name {
      get {
        return string.Format("Feynman 3.74 Goldstein 2*pi*d**(3/2)/sqrt(G*(m1+m2)) | {0} samples", trainingSamples);
      }
    }

    protected override string TargetVariable { get { return "t"; } }
    protected override string[] VariableNames { get { return new[] {"d", "G", "m1", "m2", "t"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"d", "G", "m1", "m2"}; } }

    public int Seed { get; private set; }

    protected override int TrainingPartitionStart { get { return 0; } }
    protected override int TrainingPartitionEnd { get { return trainingSamples; } }
    protected override int TestPartitionStart { get { return trainingSamples; } }
    protected override int TestPartitionEnd { get { return trainingSamples + testSamples; } }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data = new List<List<double>>();
      var d    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 3).ToList();
      var G    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 3).ToList();
      var m1   = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 3).ToList();
      var m2   = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 3).ToList();

      var t = new List<double>();

      data.Add(d);
      data.Add(G);
      data.Add(m1);
      data.Add(m2);
      data.Add(t);

      for (var i = 0; i < d.Count; i++) {
        var res = 2 * Math.PI * Math.Pow(d[i], (3.0 / 2)) / Math.Sqrt(G[i] * (m1[i] + m2[i]));
        t.Add(res);
      }

      return data;
    }
  }
}