using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class FeynmanBonus9 : FeynmanDescriptor {
    private readonly int testSamples;
    private readonly int trainingSamples;

    public FeynmanBonus9() : this((int) DateTime.Now.Ticks, 10000, 10000) { }

    public FeynmanBonus9(int seed) {
      Seed            = seed;
      trainingSamples = 10000;
      testSamples     = 10000;
    }

    public FeynmanBonus9(int seed, int trainingSamples, int testSamples) {
      Seed                 = seed;
      this.trainingSamples = trainingSamples;
      this.testSamples     = testSamples;
    }

    public override string Name {
      get {
        return string.Format(
          "Feynman Gravitational wave ratiated power -32/5*G**4/c**5*(m1*m2)**2*(m1+m2)/r**5 | {0} samples",
          trainingSamples);
      }
    }

    protected override string TargetVariable { get { return "Pwr"; } }
    protected override string[] VariableNames { get { return new[] {"G", "c", "m1", "m2", "r", "Pwr"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"G", "c", "m1", "m2", "r"}; } }

    public int Seed { get; private set; }

    protected override int TrainingPartitionStart { get { return 0; } }
    protected override int TrainingPartitionEnd { get { return trainingSamples; } }
    protected override int TestPartitionStart { get { return trainingSamples; } }
    protected override int TestPartitionEnd { get { return trainingSamples + testSamples; } }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data = new List<List<double>>();
      var G    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 2).ToList();
      var c    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 2).ToList();
      var m1   = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var m2   = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var r    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 2).ToList();

      var Pwr = new List<double>();

      data.Add(G);
      data.Add(c);
      data.Add(m1);
      data.Add(m2);
      data.Add(r);
      data.Add(Pwr);

      for (var i = 0; i < G.Count; i++) {
        var res = -32.0 / 5 * Math.Pow(G[i], 4) / Math.Pow(c[i], 5) * Math.Pow((m1[i] * m2[i]), 2) * (m1[i] + m2[i]) /
                  Math.Pow(r[i], 5);
        Pwr.Add(res);
      }

      return data;
    }
  }
}